﻿DeclareModule OWire

  ;-DeclareModule

  Enumeration OW_ErrCode
    #OW_Err_OK        ; Ошибок нет.
    #OW_Err_Port      ; Нет открытого порта (для OpenPort() - ошибка открытия порта).
    #OW_Err_NoDevice  ; Не найдены 1-Wire устройства.
    #OW_Err_IO        ; Ошибка при обмене данными с 1-Wire устройством.
    #OW_Err_BaudRate  ; Ошибка при изменении скорости порта.
    #OW_Err_Search    ; Ошибка происходит если во время поиска ни одно 1-Wire устройство не ответило.
    #OW_Err_BadParam  ; Неккоректный параметр.
    #OW_Err_NoData    ; Нет данных.
  EndEnumeration
  
  #OW_PortOnly = -10 ; Идентификатор порта по умолчанию.
  
  
  #OW_Mode_CPU_MinUsage = 1  ; Уменьшить загрузку процессора (снизится скорость обмена данными).
  #OW_Mode_Invert       = 2  ; Инвертирование сигналов DTR и RTS.
  #OW_Mode_NoControlPin = 4  ; Не изменять состояние управляющих выходов DTR и RTS.
  
  Structure OW_RAM ; Копия содержимого ОЗУ датчика DS18B20 ($BE).
    L_Termo.a       ; Младший байт температуры
    H_Termo.a       ; Старший байт температуры
    Max_Termo.a     ; Верхний предел температуры
    Min_Termo.a     ; Нижний предел температуры
    Config.a        ; Байт конфигурации
    x.a[3]          ; Резерв 3 байта (в датчике не реализовано)
    CRC.a           ; Контрольная сумма
  EndStructure
;     
  Structure OW_Buff
    Byte.a[0]
  EndStructure
  
  Structure OW_SN  ; Идентификатор датчика ($33).
    Type.a          ; Тип устройства (код семейства)
    Number.a[6]     ; Серийный номер.
    CRC.a           ; Контрольная сумма.
  EndStructure
  
  
  Declare OW_GetMode()
  Declare OW_SetMode(Mode)
  Declare OW_GetPortDefID()
  Declare OW_GetLastError() ; Код ошибки (см. константы #OW_Err_xxxx)
  Declare OW_SetLastError(ErrCode)
  Declare OW_ClosePort(Port=#OW_PortOnly) ; Закрытие порта.
  Declare OW_OpenPort(ComPort.s, Port=#OW_PortOnly) ; Открытие порта.
  Declare OW_PortID(Port=#OW_PortOnly)
  Declare OW_Reset(Port=#OW_PortOnly)   ; Сброс датчика.
  Declare.a OW_ByteRW(Byte.a=$FF, Port=#OW_PortOnly) ; Чтение/запись байта.
  Declare OW_ReadMultiByte(*Buff.OW_Buff, Count, Port=#OW_PortOnly) ; Чтение массива байт.
  Declare OW_WriteMultiByte(*Buff.OW_Buff, Count, Port=#OW_PortOnly) ; Запись массива байт.
  Declare.a OW_GetCRC(*Buff, Count)       ; Вычислене контрольной суммы.
    
  ; Термометр передает 8-битный код семейства (10h для DS18S20 и 28h для DS18B20),
  ; затем 48-битный серийный номер, а затем 8-битную CRC для проверки правильности принятой информации.
  Declare OW_ReadROM(*Info.OW_SN, Port=#OW_PortOnly)
  
  ; Эта процедура позволяет адресовать на шине конкретный термометр.
  ; Необходимо передать 64-битный код датчика (считанный процедурой ReadROM()),
  ; и только тот, который имеет такой код, будет «откликаться» до следующего импульса сброса.
  Declare OW_SelectROM(*Info.OW_SN, Port=#OW_PortOnly)
  
  ; Вызов процедуры позволит пропустить процедуру сравнения серийного номера
  ; и тем самым сэкономить время в системах, где на шине имеется всего одно устройство.
  Declare OW_SkipROM(Port=#OW_PortOnly)
  
  ; Поиск датчиков, идентификаторы которых будут помещены в список.
  Declare OW_SearchROM_Reset()
  Declare OW_SearchROM_Enum(*Sensor.OW_SN, Port=#OW_PortOnly)
  
  ; Поиск датчиков у которых результат последнего измерения температуры
  ; выходит за предустановленные пределы TH и TL.
  Declare OW_SearchAlarm_Reset()
  Declare OW_SearchAlarm_Enum(*Sensor.OW_SN, Port=#OW_PortOnly)
  
  ; Чтение данных из промежуточного ОЗУ датчика.
  Declare OW_ReadRAM(*Info.OW_RAM, Port=#OW_PortOnly)
  
  ; Запись данных в промежуточное ОЗУ датчика
  Declare OW_WriteRAM(*Info.OW_Buff, Port=#OW_PortOnly)
  
  ; Копирование данных из промежуточного ОЗУ в энергонезависимую память датчика.
  ; Эта операция требует около 10мс.
  Declare OW_CopyRAM(Port=#OW_PortOnly)
  
  ; Запуск процесса преобразования температуры.
  Declare OW_ConvertT(Wait=0, Port=#OW_PortOnly)
  
  ; Эта процедура действует обратным образом по отношению к процедуре OW_CopyRAM(),
  ; т.е. она позволяет считать байты TH и TL из энергонезависимой памяти в промежуточное ОЗУ.
  Declare OW_Recall(Port=#OW_PortOnly)
  
  ; Проверка, использует ли 1-Wire устройство паразитное питание.
  Declare OW_ReadPowerSupply(Port=#OW_PortOnly)
  
EndDeclareModule

Module OWire

EnableExplicit

;- Private

#wbf_WaitByte  = 1
#wbf_ClearByte = 2
#wbf_Slot      = 4

CompilerIf #PB_Compiler_OS = #PB_OS_Linux
  #NCCS = 32
  #TCSANOW = 0
  
  Structure termios Align #PB_Structure_AlignC
    c_iflag.l
    c_oflag.l
    c_cflag.l
    c_lflag.l
    c_line.a
    c_cc.a[#NCCS]
    c_ispeed.l
    c_ospeed.l
  EndStructure
  
  Global gModuleModeLin=0
  
CompilerEndIf

Structure sSearch
  LastDiscrepancy.l
  Done.a
  Search.a[8]
EndStructure

CompilerIf #PB_Compiler_Thread
  Threaded tLastErr=0, tSearchROM.sSearch, tSearchAlarm.sSearch
CompilerElse
  Global tLastErr=0, tSearchROM.sSearch, tSearchAlarm.sSearch
CompilerEndIf

Global gCurrent_ComPort_ID=-2, gModuleMode=0

Macro TestPortID(pID)
  If pID=#OW_PortOnly
    pID=gCurrent_ComPort_ID
  EndIf
EndMacro 

Macro Odd(xx)
  ((xx) & 1)
EndMacro

Procedure ClearInBuf(Port) ; Очистка приёмного буфера.
  Protected Count, *Mem
  
  Count = AvailableSerialPortInput(Port)
  If Count>0
    *Mem=AllocateMemory(Count+2, #PB_Memory_NoClear)
    If *Mem
      ReadSerialPortData(Port, *Mem, Count)
      FreeMemory(*Mem)
    EndIf
  EndIf
  
EndProcedure

Procedure TestPort(Port)
  
  TestPortID(Port)
  
  If IsSerialPort(Port)
    tLastErr=#OW_Err_OK
    ClearInBuf(Port)
  Else
    tLastErr=#OW_Err_Port
  EndIf
  
  ProcedureReturn Port
EndProcedure

Procedure SetBaudRate(Port, Rate)
  Protected Result=#False, PortID

  If IsSerialPort(Port)
    
    PortID = SerialPortID(Port)
    
    CompilerIf #PB_Compiler_OS = #PB_OS_Windows
      
      Protected dcb.DCB
      If PortID
        If GetCommState_(PortID, @dcb)  
          dcb\BaudRate  = Rate ; Требуемая скорость.
          If gModuleMode & #OW_Mode_NoControlPin = 0
            dcb\fbits & ~%0010000000100000
            If gModuleMode & #OW_Mode_Invert = 0
              dcb\fbits |  %0001000000010000
            Else
              dcb\fbits & ~%0001000000010000
            EndIf
          EndIf
          If SetCommState_(PortID, @dcb)
            Result = #True
          EndIf
        EndIf
      EndIf
      
    CompilerElseIf #PB_Compiler_OS = #PB_OS_Linux
      
      Protected options.termios, Sp
      
      If Rate=9600 Or Rate=115200
        
        Select Rate
          Case 9600   : Sp = 13
          Case 115200 : Sp = 4098
        EndSelect
        
        If tcgetattr_(PortID, @options) = 0   ; читает параметры порта.
          If cfsetispeed_(@options, Sp) = 0   ; установка скорости порта.
            If cfsetospeed_(@options, Sp) = 0 ; установка скорости порта.
              If tcsetattr_(PortID, #TCSANOW, @options) = 0
                Result=#True
              EndIf
            EndIf
          EndIf
        EndIf
        
      EndIf
      
      If gModuleMode & #OW_Mode_NoControlPin = 0
        If gModuleMode&#OW_Mode_Invert <> gModuleModeLin&#OW_Mode_Invert
          If gModuleMode & #OW_Mode_Invert=0
            SetSerialPortStatus(Port, #PB_SerialPort_DTR, 1)
            SetSerialPortStatus(Port, #PB_SerialPort_RTS, 1)
          Else
            SetSerialPortStatus(Port, #PB_SerialPort_DTR, 0)
            SetSerialPortStatus(Port, #PB_SerialPort_RTS, 0)
          EndIf
        EndIf
        gModuleModeLin=gModuleMode
      EndIf
      
    CompilerEndIf
    
    If Result=#False : tLastErr=#OW_Err_BaudRate : EndIf
  EndIf
    
  ProcedureReturn Result
EndProcedure


Procedure.a ReadBit(Port, Flag=#wbf_WaitByte|#wbf_ClearByte|#wbf_Slot)
  Protected Bit.a, x.l
  
  tLastErr=#OW_Err_OK
  
  If Flag&#wbf_ClearByte
    ClearInBuf(Port)
  EndIf
  
  Bit=$FF
  If Flag&#wbf_Slot
    If WriteSerialPortData(Port, @Bit, 1)<>1
      tLastErr=#OW_Err_IO
      ProcedureReturn 0
    EndIf
  EndIf
  
  If Flag&#wbf_WaitByte
    x = ElapsedMilliseconds()
    Repeat
      If gModuleMode & #OW_Mode_CPU_MinUsage : Delay(1) : EndIf
    Until AvailableSerialPortInput(Port)>0 Or Int(ElapsedMilliseconds()-x)>=36
  EndIf
  
  If AvailableSerialPortInput(Port)=1
    x=0
    If ReadSerialPortData(Port, @x, 1) = 1
      Bit = Bool(x=$FF)
    Else
      tLastErr=#OW_Err_IO
    EndIf
  Else
    tLastErr=#OW_Err_IO
  EndIf
  
  ProcedureReturn Odd(Bit)
EndProcedure


Procedure WriteBit(Bit.a, Port, Flag=#wbf_WaitByte|#wbf_ClearByte)
  Bit=Odd(Bit)*$FF
  WriteSerialPortData(Port, @Bit, 1)
  If Flag & #wbf_WaitByte
    Protected x.l = ElapsedMilliseconds()
    Repeat
      If gModuleMode & #OW_Mode_CPU_MinUsage : Delay(1) : EndIf
    Until AvailableSerialPortInput(Port)>0 Or Int(ElapsedMilliseconds()-x)>=36
    If Flag & #wbf_ClearByte And AvailableSerialPortInput(Port)>0
      ReadSerialPortData(Port, @Bit, 1)
    EndIf
  EndIf
EndProcedure

; Код на основе материала. http://www.maximintegrated.com/app-notes/index.mvp/id/187
Procedure SearchDevice(*Sensor.OW_SN, *Search.sSearch, Port, Code.a)

  Protected Result=#False
  Protected.l id_bit_number, last_zero
  Protected.l rom_byte_number
  Protected.l id_bit, cmp_id_bit
  Protected.a rom_byte_mask, search_direction
  
  If *Search\Done = #False
    
    ClearInBuf(Port)
    
    
    id_bit_number = 1
    last_zero = 0
    rom_byte_number = 0
    rom_byte_mask = 1
    
    If OW_Reset(Port)
      
      OW_WriteMultiByte(@Code, 1, Port)
      
      If tLastErr=#OW_Err_OK
        
        Repeat
          
          id_bit = ReadBit(Port)
          If tLastErr<>#OW_Err_OK : Break : EndIf
          
          cmp_id_bit = ReadBit(Port)
          If tLastErr<>#OW_Err_OK : Break : EndIf
          
          If id_bit And cmp_id_bit ; Ни один датчик не ответил!
            
            tLastErr=#OW_Err_Search
            Break
          ElseIf id_bit <> cmp_id_bit
            search_direction = id_bit
          Else  ; Произошла коллизия (больше одного датчика и у них разные ID).
            
            If id_bit_number < *Search\LastDiscrepancy
              search_direction = Bool((*Search\Search[rom_byte_number] & rom_byte_mask))
            Else
              search_direction = Bool(id_bit_number=*Search\LastDiscrepancy)
            EndIf
            
            If search_direction = 0
              last_zero = id_bit_number
            EndIf
            
          EndIf
          
          If search_direction
            *Search\Search[rom_byte_number] | rom_byte_mask
          Else
            *Search\Search[rom_byte_number] & ~rom_byte_mask
          EndIf
          
          WriteBit(search_direction, Port)
          
          id_bit_number+1
          rom_byte_mask<<1
          
          If rom_byte_mask=0
            rom_byte_number+1
            rom_byte_mask = 1
          EndIf
          
        Until rom_byte_number >= 8
        
        If Not id_bit_number < 65
          *Search\LastDiscrepancy = last_zero
          If *Search\LastDiscrepancy = 0
            *Search\Done = #True
          EndIf
          
          CopyMemory(@*Search\Search[0], *Sensor, 8)
          Result=#True
        EndIf
        
      EndIf
    EndIf
    
  Else
    tLastErr=#OW_Err_NoData
  EndIf
  
  ProcedureReturn Result
EndProcedure



;-Public

Procedure OW_GetMode()
  ProcedureReturn gModuleMode
EndProcedure

Procedure OW_SetMode(Mode)
  CompilerIf #PB_Compiler_OS = #PB_OS_Linux
    gModuleModeLin=gModuleMode
  CompilerEndIf
  gModuleMode = Mode
EndProcedure

Procedure OW_GetPortDefID()
  ProcedureReturn gCurrent_ComPort_ID
EndProcedure

Procedure OW_GetLastError()
  ProcedureReturn tLastErr
EndProcedure

Procedure OW_SetLastError(ErrCode)
  tLastErr = ErrCode
EndProcedure

Procedure OW_ClosePort(Port=#OW_PortOnly)
  Protected Result = #False, OldPort
  
  OldPort=Port
  TestPortID(Port)
  
  If IsSerialPort(Port)
    If gModuleMode & #OW_Mode_NoControlPin = 0
      If gModuleMode & #OW_Mode_Invert = 0
        SetSerialPortStatus(Port, #PB_SerialPort_DTR, 0)
        SetSerialPortStatus(Port, #PB_SerialPort_RTS, 0)
      Else
        SetSerialPortStatus(Port, #PB_SerialPort_DTR, 1)
        SetSerialPortStatus(Port, #PB_SerialPort_RTS, 1)
      EndIf
    EndIf
    CloseSerialPort(Port)
    If OldPort=#OW_PortOnly
      gCurrent_ComPort_ID=-2
    EndIf
    Result = #True
    tLastErr = #OW_Err_OK
  Else
    tLastErr = #OW_Err_Port
  EndIf
  
  ProcedureReturn Result
EndProcedure

Procedure OW_OpenPort(ComPort.s, Port=#OW_PortOnly)
  Protected ID=0, PortID=-2
  
  If Port>=0 And Port<1000
    OW_ClosePort(Port)
    PortID=Port
  ElseIf Port=#OW_PortOnly 
    OW_ClosePort(Port)
    PortID=#PB_Any
  ElseIf Port=#PB_Any
    PortID=#PB_Any
  Else  
    tLastErr=#OW_Err_BadParam ; Неправильный идентификатор порта
    ProcedureReturn 0
  EndIf
  
  ID = OpenSerialPort(PortID, ComPort, 115200, #PB_SerialPort_NoParity,
                      8, 1, #PB_SerialPort_NoHandshake, 255, 255)
  If ID
    
    If Port<0
      If Port=#OW_PortOnly
        gCurrent_ComPort_ID = ID
      EndIf
      Port=ID
    EndIf
    
    If gModuleMode & #OW_Mode_NoControlPin = 0
      If gModuleMode & #OW_Mode_Invert = 0
        SetSerialPortStatus(Port, #PB_SerialPort_DTR, 1)
        SetSerialPortStatus(Port, #PB_SerialPort_RTS, 1)
      Else
        SetSerialPortStatus(Port, #PB_SerialPort_DTR, 0)
        SetSerialPortStatus(Port, #PB_SerialPort_RTS, 0)
      EndIf
      Delay(20)
    EndIf
    ;SerialPortTimeouts(Port, 10, 10, 2, 2, 10)
    OW_ByteRW($FF, Port)
    ClearInBuf(Port)
    
    tLastErr = #OW_Err_OK
    
  Else
    tLastErr=#OW_Err_Port
  EndIf
  
  ProcedureReturn ID
EndProcedure


Procedure OW_PortID(Port=#OW_PortOnly)
  Protected ID=0
  
  Port=TestPort(Port)
  If tLastErr = #OW_Err_OK
    ID = SerialPortID(Port)
  EndIf
  
  ProcedureReturn ID
EndProcedure

Procedure OW_Reset(Port=#OW_PortOnly) ; Подаём команду "Сброс" датчику температуры.
  Protected x.l, Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr = #OW_Err_OK
    If SetBaudRate(Port, 9600)
      
      ClearInBuf(Port)
      
      x=$F0
      If WriteSerialPortData(Port, @x, 1)=1
        
        x = ElapsedMilliseconds()
        Repeat
          Delay(2)
        Until AvailableSerialPortInput(Port)>0 Or Int(ElapsedMilliseconds()-x)>=32
        
        If AvailableSerialPortInput(Port)=1
          x=0
          If ReadSerialPortData(Port, @x, 1) = 1
            Result=Bool(x<>$F0)
            If Result=#False : tLastErr=#OW_Err_NoDevice : EndIf
          Else
            Goto M1:
          EndIf
        Else
          M1:
          tLastErr=#OW_Err_IO
          ClearInBuf(Port)
        EndIf
        
      Else
        tLastErr=#OW_Err_IO
      EndIf
      
      SetBaudRate(Port, 115200)
      
    EndIf
  EndIf
  
  ProcedureReturn Result
EndProcedure


Procedure.a OW_ByteRW(Byte.a=$FF, Port=#OW_PortOnly)
  Protected Result.a, x.a, i
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK  
    For i= 1 To 8

      WriteBit(Odd(Byte), Port, #wbf_WaitByte)
      Byte>>1
      
      x=ReadBit(Port, 0)
      If tLastErr=#OW_Err_OK
        Result>>1
        Result|(x<<7)
      Else
        Result = $FF
        Break
      EndIf

    Next i
  EndIf

  ProcedureReturn Result
EndProcedure

Procedure OW_ReadMultiByte(*Buff.OW_Buff, Count, Port=#OW_PortOnly) ; Чтение массива байт.
  Protected Result=#False, Time, WaitTime
  Protected RealCount, i, x, Byte.a, Pos, *Mem.OW_Buff
  
  If *Buff And Count>0 And Count<1000000
       
    Port=TestPort(Port)
    
    If tLastErr=#OW_Err_OK
      
      RealCount=Count*8

      WaitTime=36+(Count*2)
      Time = ElapsedMilliseconds()
      
      *Mem=AllocateMemory(RealCount+2, #PB_Memory_NoClear)
      
      If *Mem
        
        FillMemory(*Mem, RealCount, $FF, #PB_Ascii)
        
        If WriteSerialPortData(Port, *Mem, RealCount)=RealCount
          
          Repeat
            If gModuleMode & #OW_Mode_CPU_MinUsage : Delay(2) : EndIf
          Until Int(ElapsedMilliseconds()-Time)>=WaitTime Or AvailableSerialPortInput(Port)>=RealCount
          
          If AvailableSerialPortInput(Port)=RealCount
            
            FillMemory(*Mem, RealCount, 0, #PB_Ascii)
            
            If ReadSerialPortData(Port, *Mem, RealCount) = RealCount
              Pos=0
              For i=0 To Count-1
                Byte=0
                For x=0 To 7
                  Byte>>1
                  Byte | (Bool(*Mem\Byte[Pos]=$FF)<<7)
                  Pos+1
                Next x
                *Buff\Byte[i]=Byte : 
              Next i
              Result=#True
            Else
              tLastErr=#OW_Err_IO
            EndIf
            
          Else
            ClearInBuf(Port)
            tLastErr=#OW_Err_IO
          EndIf
          
        Else
          tLastErr=#OW_Err_IO
        EndIf
        
        FreeMemory(*Mem)
        
      Else
        tLastErr=#OW_Err_BadParam
      EndIf
      
    Else
      tLastErr=#OW_Err_Port
    EndIf
  Else
    tLastErr=#OW_Err_BadParam
  EndIf
  
  ProcedureReturn Result
EndProcedure

Procedure OW_WriteMultiByte(*Buff.OW_Buff, Count, Port=#OW_PortOnly) ; Запись массива байт.
  Protected Result=#False, i, x, Byte.a, RealCount
  Protected Pos, *Mem.OW_Buff
  
  If (*Buff And Count>0 And Count<1000000) Or (*Buff=0 And Count=1)
    Port=TestPort(Port)
    
    If tLastErr=#OW_Err_OK 
      
      
      RealCount=Count*8
      
      *Mem=AllocateMemory(RealCount+2, #PB_Memory_NoClear)
      If *Mem
        
        Pos=0
        For x=0 To Count-1
          
          If Count=1 And *Buff&~$FF=0
            Byte=*Buff
          Else
            Byte=*Buff\Byte[x]
          EndIf
          For i=1 To 8
            *Mem\Byte[Pos]=Odd(Byte)*$FF
            Pos+1
            Byte>>1
          Next i
        Next x
        
        If WriteSerialPortData(Port, *Mem, RealCount)=RealCount
          
          x=36+(Count*2)
          i = ElapsedMilliseconds()
          
          Repeat
            If gModuleMode & #OW_Mode_CPU_MinUsage : Delay(2) : EndIf
          Until Int(ElapsedMilliseconds()-i)>=x Or AvailableSerialPortInput(Port)>=RealCount
          
          If AvailableSerialPortInput(Port)=RealCount
            If ReadSerialPortData(Port, *Mem, RealCount) = RealCount
              Result=#True
            Else
              tLastErr=#OW_Err_IO
            EndIf
          Else
            ClearInBuf(Port)
            tLastErr=#OW_Err_IO
          EndIf
          
        Else
          ClearInBuf(Port)
          tLastErr=#OW_Err_IO
        EndIf
        
        FreeMemory(*Mem)
        
      Else
        tLastErr=#OW_Err_BadParam
      EndIf
      
    EndIf
    
  Else
    tLastErr=#OW_Err_BadParam
  EndIf
  
  ProcedureReturn Result
EndProcedure


Procedure.a OW_GetCRC(*Buff.OW_Buff, Count)
  Protected CRC.a, Byte.a, i, x
  
  If *Buff And Count>0
    
    tLastErr=#OW_Err_OK
    
    Count-1
    For i=0 To Count
      Byte=*Buff\Byte[i]
      For x=1 To 8
        If (Byte ! CRC) & 1
          CRC = ((CRC!$18)>>1)|$80
        Else
          CRC >> 1
        EndIf
        Byte>>1
      Next x
    Next i
    
  Else
    tLastErr=#OW_Err_BadParam
  EndIf
  
  ProcedureReturn CRC
EndProcedure

; Датчик передает 8-битный код семейства (10h для DS18S20 и 28h для DS18B20),
; затем 48-битный серийный номер, а затем 8-битную CRC для проверки правильности принятой информации.
Procedure OW_ReadROM(*Info.OW_Buff, Port=#OW_PortOnly)
  Protected i, Result=#False
  
  Port=TestPort(Port)
   
  If tLastErr=#OW_Err_OK
    If *Info
      If OW_Reset(Port)
        If OW_WriteMultiByte($33, 1, Port)
          If OW_ReadMultiByte(@*Info\Byte[0], 8, Port)
            Result=#True
          EndIf
        EndIf
      EndIf
    Else
      tLastErr=#OW_Err_BadParam
    EndIf
  EndIf
  
  ProcedureReturn Result
EndProcedure

; Эта процедура позволяет адресовать на шине конкретный датчик температуры.
; Необходимо передать 64-битный код датчика (считанный процедурой OW_ReadROM()),
; и только тот, который имеет такой код, будет «откликаться» до следующего импульса сброса.
Procedure OW_SelectROM(*Info.OW_Buff, Port=#OW_PortOnly)
  Protected i, Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK
    If *Info
      If OW_Reset(Port)      
        If OW_WriteMultiByte($55, 1, Port)
          If OW_WriteMultiByte(*Info, 8, Port)
            Result=#True
          EndIf
        EndIf
      EndIf
    Else
      tLastErr=#OW_Err_BadParam
    EndIf
  EndIf
  
  ProcedureReturn Result
EndProcedure

; Вызов процедуры позволит пропустить процедуру сравнения серийного номера
; и тем самым сэкономить время в системах, где на шине имеется всего одно устройство.
Procedure OW_SkipROM(Port=#OW_PortOnly)
  Protected Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK
    If OW_Reset(Port)
      If OW_WriteMultiByte($CC, 1, Port)
        Result=#True
      EndIf
    EndIf
  EndIf
  
  ProcedureReturn Result
EndProcedure


Procedure OW_SearchROM_Reset()
  tLastErr=#OW_Err_OK
  FillMemory(@tSearchROM, SizeOf(sSearch))
  ProcedureReturn #True
EndProcedure

Procedure OW_SearchROM_Enum(*Sensor.OW_SN, Port=#OW_PortOnly)
  Protected Result=#False
  
  If *Sensor
    Port=TestPort(Port)
    If tLastErr=#OW_Err_OK
      Result=SearchDevice(*Sensor, @tSearchROM, Port, $F0)
    EndIf
  Else
    tLastErr=#OW_Err_BadParam
  EndIf
    
  ProcedureReturn Result
EndProcedure


Procedure OW_SearchAlarm_Reset()
  tLastErr=#OW_Err_OK
  FillMemory(@tSearchAlarm, SizeOf(sSearch))
  ProcedureReturn #True
EndProcedure

Procedure OW_SearchAlarm_Enum(*Sensor.OW_SN, Port=#OW_PortOnly)
  Protected Result=#False
  
  If *Sensor
    Port=TestPort(Port)
    If tLastErr=#OW_Err_OK
      Result=SearchDevice(*Sensor, @tSearchAlarm, Port, $EC)
    EndIf
  Else
    tLastErr=#OW_Err_BadParam
  EndIf
    
  ProcedureReturn Result
EndProcedure


; Чтение данных из промежуточного ОЗУ датчика.
Procedure OW_ReadRAM(*Info.OW_Buff, Port=#OW_PortOnly)
  Protected i, Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK
    If *Info
      If OW_WriteMultiByte($BE, 1, Port)
        If OW_ReadMultiByte(*Info, 9, Port) : Result=#True : EndIf
      EndIf
    Else
      tLastErr=#OW_Err_BadParam
    EndIf
  EndIf
  
  ProcedureReturn Result
EndProcedure

; Запись данных в промежуточное ОЗУ датчика
Procedure OW_WriteRAM(*Info.OW_Buff, Port=#OW_PortOnly)
  Protected i, Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK
    If *Info
      If OW_WriteMultiByte($4E, 1, Port)
        
        For i=0 To 2
          OW_ByteRW(*Info\Byte[i], Port)
          If tLastErr<>#OW_Err_OK : Break : EndIf
        Next
        
        If tLastErr=#OW_Err_OK : Result=#True : EndIf
        
      EndIf
    Else
      tLastErr=#OW_Err_BadParam
    EndIf
  EndIf
   
  ProcedureReturn Result
EndProcedure

; Копирование данных из промежуточного ОЗУ в энергонезависимую память датчика.
; Это требует около 10мс.
Procedure OW_CopyRAM(Port=#OW_PortOnly)
  Protected Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK
    If OW_WriteMultiByte($48, 1, Port) 
      Delay(20)
      Result=#True 
    EndIf
  EndIf
    
  ProcedureReturn Result
EndProcedure

; Запуск процесса преобразования температуры.
Procedure OW_ConvertT(Wait=0, Port=#OW_PortOnly)
  Protected Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK
    If OW_WriteMultiByte($44, 1, Port)
      Result=#True
      If Wait>0
        Delay(Wait)
      EndIf
    EndIf
  EndIf
    
  ProcedureReturn Result
EndProcedure

; Эта процедура действует обратным образом по отношению к процедуре OW_CopyRAM(),
; т.е. она позволяет считать байты TH и TL из энергонезависимой памяти в промежуточное ОЗУ.
Procedure OW_Recall(Port=#OW_PortOnly)
  Protected Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK
    If OW_WriteMultiByte($B8, 1, Port) : Result=#True : EndIf
  EndIf
    
  ProcedureReturn Result
EndProcedure

; Проверка, использует ли 1-Wire устройство "паразитное" питание.
Procedure OW_ReadPowerSupply(Port=#OW_PortOnly)
  Protected Result=#False
  
  Port=TestPort(Port)
  
  If tLastErr=#OW_Err_OK
    If OW_WriteMultiByte($B4, 1, Port)
      Result=ReadBit(Port)
    EndIf
  EndIf
  
  ProcedureReturn Result
EndProcedure

EndModule


; IDE Options = PureBasic 5.40 LTS (Windows - x86)
; Folding = -------
; EnableXP